import { Observable } from "../Misc/observable.js";
import type { Nullable } from "../types.js";
import type { AbstractEngine } from "./abstractEngine.js";
import type { Scene } from "../scene.js";
/**
 * The engine store class is responsible to hold all the instances of Engine and Scene created
 * during the life time of the application.
 */
export declare class EngineStore {
    /** Gets the list of created engines */
    static Instances: AbstractEngine[];
    /**
     * Notifies when an engine was disposed.
     * Mainly used for static/cache cleanup
     */
    static OnEnginesDisposedObservable: Observable<AbstractEngine>;
    /** @internal */
    static _LastCreatedScene: Nullable<Scene>;
    /**
     * Gets the latest created engine
     */
    static get LastCreatedEngine(): Nullable<AbstractEngine>;
    /**
     * Gets the latest created scene
     */
    static get LastCreatedScene(): Nullable<Scene>;
    /**
     * Gets or sets a global variable indicating if fallback texture must be used when a texture cannot be loaded
     */
    static UseFallbackTexture: boolean;
    /**
     * Texture content used if a texture cannot loaded
     */
    static FallbackTexture: string;
}
