import { NodeMaterialBlock } from "../../nodeMaterialBlock.js";
import type { NodeMaterialBuildState } from "../../nodeMaterialBuildState.js";
import type { NodeMaterialConnectionPoint } from "../../nodeMaterialBlockConnectionPoint.js";
import type { AbstractMesh } from "../../../../Meshes/abstractMesh.js";
import type { NodeMaterial, NodeMaterialDefines } from "../../nodeMaterial.js";
import type { Effect } from "../../../effect.js";
import type { Mesh } from "../../../../Meshes/mesh.js";
/**
 * Block used to add morph targets support to vertex shader
 */
export declare class MorphTargetsBlock extends NodeMaterialBlock {
    private _repeatableContentAnchor;
    /**
     * Create a new MorphTargetsBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the position input component
     */
    get position(): NodeMaterialConnectionPoint;
    /**
     * Gets the normal input component
     */
    get normal(): NodeMaterialConnectionPoint;
    /**
     * Gets the tangent input component
     */
    get tangent(): NodeMaterialConnectionPoint;
    /**
     * Gets the uv input component
     */
    get uv(): NodeMaterialConnectionPoint;
    /**
     * Gets the uv2 input component
     */
    get uv2(): NodeMaterialConnectionPoint;
    /**
     * Gets the color input component
     */
    get color(): NodeMaterialConnectionPoint;
    /**
     * Gets the position output component
     */
    get positionOutput(): NodeMaterialConnectionPoint;
    /**
     * Gets the normal output component
     */
    get normalOutput(): NodeMaterialConnectionPoint;
    /**
     * Gets the tangent output component
     */
    get tangentOutput(): NodeMaterialConnectionPoint;
    /**
     * Gets the uv output component
     */
    get uvOutput(): NodeMaterialConnectionPoint;
    /**
     * Gets the uv2 output component
     */
    get uv2Output(): NodeMaterialConnectionPoint;
    /**
     * Gets the color output component
     */
    get colorOutput(): NodeMaterialConnectionPoint;
    initialize(state: NodeMaterialBuildState): void;
    private _initShaderSourceAsync;
    autoConfigure(material: NodeMaterial, additionalFilteringInfo?: (node: NodeMaterialBlock) => boolean): void;
    prepareDefines(defines: NodeMaterialDefines, nodeMaterial: NodeMaterial, mesh?: AbstractMesh): void;
    bind(effect: Effect, nodeMaterial: NodeMaterial, mesh?: Mesh): void;
    replaceRepeatableContent(vertexShaderState: NodeMaterialBuildState, defines: NodeMaterialDefines, mesh?: AbstractMesh): void;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}
